"""Set of constants."""

MAX_CHUNK_SIZE = 3900
MAX_CHUNK_OVERLAP = 200
NUM_OUTPUTS = 256


INDEX_STRUCT_KEY = "index_struct"
DOCSTORE_KEY = "docstore"
VECTOR_STORE_KEY = "vector_store"
ADDITIONAL_QUERY_CONTEXT_KEY = "query_context"
ALL_INDICES_KEY = "all_indices"
ROOT_INDEX_ID_KEY = "root_index_id"

TYPE_KEY = "__type__"
DATA_KEY = "__data__"
